const http = require('http');
const Koa = require('koa');
const express = require('express');
const hopper = require('./');

// ------------- KOA ---------------
const koaApp = new Koa();

const hopKoa = hopper();
hopKoa.addField('type', () => 'KOA');

koaApp.use(hopKoa);
koaApp.use(async ctx => {
  ctx.body = 'Hello World';
});

koaApp.listen(4000);
// ------------- KOA ---------------

// ------------- EXPRESS -----------
const expressApp = express();
const hopExpress = hopper({ type: 'express' });
hopExpress.addField('type', () => 'EXPRESS');

expressApp.use(hopExpress);
expressApp.get('/', (req, res) => res.send('Hello World!'));

expressApp.listen(4001);
// ------------- EXPRESS -----------

// ------ CUSTOM Middleware --------
const middlewareCreator = ({ fieldInterpreters, resolver, handler }) => (req, res) => {
  const entry = resolver(fieldInterpreters, req, res);
  handler(entry);
};
const hopCustom = hopper({ middlewareCreator });
hopCustom.addField('type', () => 'CUSTOM');
hopCustom.addField('method', req => req.method);
hopCustom.addField('headers', req => req.headers);

const server = http.createServer((req, res) => {
  hopCustom(req, res);
  res.end();
});
server.listen(4002);
// ------ CUSTOM Middleware --------

// ------------ RESOLVER -----------
const resolverKoa = new Koa();
function myresolver(fieldInterpreters, ctx) {
  const result = {
    method: fieldInterpreters.method(ctx),
    type: fieldInterpreters.type(),
  };
  return result;
}
const resolverHopper = hopper({ resolver: myresolver });
resolverHopper.addField('type', () => 'Custom Resolver');

resolverKoa.use(resolverHopper);
resolverKoa.use(async ctx => {
  ctx.body = 'Hello World';
});

resolverKoa.listen(4003);
// ------------ RESOLVER -----------

// ------------ HANDLER ------------
const handlerKoa = new Koa();
function myHandler(entry) {
  console.log(`${entry.method} ${entry.url} (${entry.status}) - ${entry.type}`);
}
const handlerHopper = hopper({ handler: myHandler });
handlerHopper.addField('type', () => 'Custom handler');
handlerKoa.use(handlerHopper);
handlerKoa.use(async ctx => {
  ctx.body = 'Hello World';
});

handlerKoa.listen(4004);
// ------------ HANDLER ------------
